package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.CharLiteral;
import org.eclipse.jdt.internal.compiler.ast.ExtendedStringLiteral;
import org.eclipse.jdt.internal.compiler.ast.Literal;
import org.eclipse.jdt.internal.compiler.ast.StringLiteralConcatenation;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.impl.StringConstant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.util.Util;

public class StringLiteral extends Literal {
  char[] source;
  
  int lineNumber;
  
  public StringLiteral(char[] token, int start, int end, int lineNumber) {
    this(start, end);
    this.source = token;
    this.lineNumber = lineNumber - 1;
  }
  
  public StringLiteral(int s, int e) {
    super(s, e);
  }
  
  public void computeConstant() {
    this.constant = StringConstant.fromValue(String.valueOf(this.source));
  }
  
  public ExtendedStringLiteral extendWith(CharLiteral lit) {
    return new ExtendedStringLiteral(this, lit);
  }
  
  public ExtendedStringLiteral extendWith(org.eclipse.jdt.internal.compiler.ast.StringLiteral lit) {
    return new ExtendedStringLiteral(this, lit);
  }
  
  public StringLiteralConcatenation extendsWith(org.eclipse.jdt.internal.compiler.ast.StringLiteral lit) {
    return new StringLiteralConcatenation(this, lit);
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (valueRequired)
      codeStream.ldc(this.constant.stringValue()); 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public TypeBinding literalType(BlockScope scope) {
    return (TypeBinding)scope.getJavaLangString();
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    output.append('"');
    for (int i = 0; i < this.source.length; i++)
      Util.appendEscapedChar(output, this.source[i], true); 
    output.append('"');
    return output;
  }
  
  public char[] source() {
    return this.source;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}
